import type { AgentMessage } from '../../agent/AgentMessage';
import type { Query } from '../../storage/StorageService';
import type { DeleteCredentialOptions } from './CredentialServiceOptions';
import type { AcceptCredentialOptions, AcceptCredentialOfferOptions, AcceptCredentialProposalOptions, AcceptCredentialRequestOptions, CreateOfferOptions, FindCredentialMessageReturn, FindCredentialOfferMessageReturn, FindCredentialProposalMessageReturn, FindCredentialRequestMessageReturn, GetFormatDataReturn, NegotiateCredentialOfferOptions, NegotiateCredentialProposalOptions, OfferCredentialOptions, ProposeCredentialOptions, SendCredentialProblemReportOptions } from './CredentialsApiOptions';
import type { CredentialFormat } from './formats';
import type { IndyCredentialFormat } from './formats/indy/IndyCredentialFormat';
import type { JsonLdCredentialFormat } from './formats/jsonld/JsonLdCredentialFormat';
import type { CredentialExchangeRecord } from './repository/CredentialExchangeRecord';
import type { CredentialService } from './services/CredentialService';
import { AgentContext } from '../../agent';
import { MessageSender } from '../../agent/MessageSender';
import { Logger } from '../../logger';
import { DidCommMessageRepository } from '../../storage/didcomm/DidCommMessageRepository';
import { ConnectionService } from '../connections/services';
import { RoutingService } from '../routing/services/RoutingService';
import { CredentialsModuleConfig } from './CredentialsModuleConfig';
import { RevocationNotificationService } from './protocol/revocation-notification/services';
import { V1CredentialService } from './protocol/v1/V1CredentialService';
import { V2CredentialService } from './protocol/v2/V2CredentialService';
import { CredentialRepository } from './repository/CredentialRepository';
export interface CredentialsApi<CFs extends CredentialFormat[], CSs extends CredentialService<CFs>[]> {
    proposeCredential(options: ProposeCredentialOptions<CFs, CSs>): Promise<CredentialExchangeRecord>;
    acceptProposal(options: AcceptCredentialProposalOptions<CFs>): Promise<CredentialExchangeRecord>;
    negotiateProposal(options: NegotiateCredentialProposalOptions<CFs>): Promise<CredentialExchangeRecord>;
    offerCredential(options: OfferCredentialOptions<CFs, CSs>): Promise<CredentialExchangeRecord>;
    acceptOffer(options: AcceptCredentialOfferOptions<CFs>): Promise<CredentialExchangeRecord>;
    declineOffer(credentialRecordId: string): Promise<CredentialExchangeRecord>;
    negotiateOffer(options: NegotiateCredentialOfferOptions<CFs>): Promise<CredentialExchangeRecord>;
    acceptRequest(options: AcceptCredentialRequestOptions<CFs>): Promise<CredentialExchangeRecord>;
    acceptCredential(options: AcceptCredentialOptions): Promise<CredentialExchangeRecord>;
    createOffer(options: CreateOfferOptions<CFs, CSs>): Promise<{
        message: AgentMessage;
        credentialRecord: CredentialExchangeRecord;
    }>;
    sendProblemReport(options: SendCredentialProblemReportOptions): Promise<CredentialExchangeRecord>;
    getAll(): Promise<CredentialExchangeRecord[]>;
    findAllByQuery(query: Query<CredentialExchangeRecord>): Promise<CredentialExchangeRecord[]>;
    getById(credentialRecordId: string): Promise<CredentialExchangeRecord>;
    findById(credentialRecordId: string): Promise<CredentialExchangeRecord | null>;
    deleteById(credentialRecordId: string, options?: DeleteCredentialOptions): Promise<void>;
    update(credentialRecord: CredentialExchangeRecord): Promise<void>;
    getFormatData(credentialRecordId: string): Promise<GetFormatDataReturn<CFs>>;
    findProposalMessage(credentialExchangeId: string): Promise<FindCredentialProposalMessageReturn<CSs>>;
    findOfferMessage(credentialExchangeId: string): Promise<FindCredentialOfferMessageReturn<CSs>>;
    findRequestMessage(credentialExchangeId: string): Promise<FindCredentialRequestMessageReturn<CSs>>;
    findCredentialMessage(credentialExchangeId: string): Promise<FindCredentialMessageReturn<CSs>>;
}
export declare class CredentialsApi<CFs extends CredentialFormat[] = [IndyCredentialFormat, JsonLdCredentialFormat], CSs extends CredentialService<CFs>[] = [V1CredentialService, V2CredentialService<CFs>]> implements CredentialsApi<CFs, CSs> {
    /**
     * Configuration for the connections module
     */
    readonly config: CredentialsModuleConfig;
    private connectionService;
    private messageSender;
    private credentialRepository;
    private agentContext;
    private didCommMessageRepository;
    private routingService;
    private logger;
    private serviceMap;
    constructor(messageSender: MessageSender, connectionService: ConnectionService, agentContext: AgentContext, logger: Logger, credentialRepository: CredentialRepository, mediationRecipientService: RoutingService, didCommMessageRepository: DidCommMessageRepository, v1Service: V1CredentialService, v2Service: V2CredentialService<CFs>, _revocationNotificationService: RevocationNotificationService, config: CredentialsModuleConfig);
    getService<PVT extends CredentialService['version']>(protocolVersion: PVT): CredentialService<CFs>;
    private getServiceForCredentialExchangeId;
}
